@@ -18,7 +18,13 @@ use paste::paste;
 
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
-#[cfg(any(feature = "f3", feature = "l4", feature = "g4", feature = "h7"))]
+#[cfg(any(
+    feature = "f3",
+    feature = "l4",
+    feature = "g4",
+    feature = "h7",
+    feature = "wb"
+))]
 use crate::pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "f4", feature = "l5")))]
@@ -268,7 +274,7 @@ pub struct Adc<R> {
     vdda_calibrated: f32,
 }
 
-// Abstract implementation of ADC functionality
+// todo: Remove this macro, and replace using a `regs` fn like you use in GPIO.
 macro_rules! hal {
     ($ADC:ident, $ADC_COMMON:ident, $adc:ident, $rcc_num:tt) => {
         impl Adc<pac::$ADC> {
@@ -828,8 +834,10 @@ macro_rules! hal {
             /// Read data from a conversion. In OneShot mode, this will generally be run right
             /// after `start_conversion`.
             pub fn read_result(&mut self) -> u16 {
-                // todo: read_volatile to be more efficient due to only reading 16 bits?
-                self.regs.dr.read().bits() as u16
+                #[cfg(feature = "l4")]
+                return self.regs.dr.read().bits() as u16;
+                #[cfg(not(feature = "l4"))]
+                return self.regs.dr.read().rdata().bits() as u16;
             }
 
             /// Take a single reading, in OneShot mode
@@ -840,7 +848,8 @@ macro_rules! hal {
 
             // todo: fn read_voltage, using vrefint and L4xx-hal style calibration?
 
-            #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+            // todo: H7 having issues with adc3 etc. Fix at least for adc1 and 2.
+            #[cfg(not(any(feature = "g0", feature = "f4", feature = "h7", feature = "l5")))]
             /// Take a one shot reading, using DMA. See L44 RM, 16.4.27: "DMA one shot mode".
             /// Note that the `channel` argument is only used on F3 and L4.
             pub unsafe fn read_dma<D>(
@@ -852,7 +861,6 @@ macro_rules! hal {
             ) where
                 D: Deref<Target = dma_p::RegisterBlock>,
             {
-
                 let (ptr, len) = (buf.as_mut_ptr(), buf.len());
                 // The software is allowed to write (dmaen and dmacfg) only when ADSTART=0 and JADSTART=0 (which
                 // ensures that no conversion is ongoing)
@@ -866,7 +874,7 @@ macro_rules! hal {
                 // L44 RM, Table 41. "DMA1 requests for each channel
                 // todo: DMA2 support.
                 #[cfg(any(feature = "f3", feature = "l4"))]
-                let channel = match self.device {
+                let dma_channel = match self.device {
                     AdcDevice::One => DmaInput::Adc1.dma1_channel(),
                     AdcDevice::Two => DmaInput::Adc2.dma1_channel(),
                     _ => panic!("DMA on ADC beyond 2 is not supported. If it is for your MCU, please submit an issue \
@@ -921,11 +929,16 @@ macro_rules! hal {
                 // • Scan sequence is stopped and reset.
                 // • The DMA is stopped.
 
+                #[cfg(feature = "h7")]
+                let len = len as u32;
+                #[cfg(not(feature = "h7"))]
+                let len = len as u16;
+
                 dma.cfg_channel(
                     dma_channel,
                     &self.regs.dr as *const _ as u32,
                     ptr as u32,
-                    len as u16,
+                    len,
                     dma::Direction::ReadFromPeriph,
                     dma::DataSize::S16,
                     dma::DataSize::S16,
