@@ -180,7 +180,7 @@ impl ApbPrescaler {
 
 #[derive(Clone, Copy, PartialEq)]
 #[repr(u8)]
-/// SAI clock input source. Sets RCC_D2CCIP1R register, SAIxSEL fields.
+/// SAI clock input source. Sets RCC_D2CCIP1R register, SAIxSEL field.
 pub enum SaiSrc {
     Pll1Q = 0b000,
     Pll2P = 0b001,
@@ -189,6 +189,16 @@ pub enum SaiSrc {
     PerClk = 0100,
 }
 
+#[derive(Clone, Copy, PartialEq)]
+#[repr(u8)]
+/// SAI clock input source. Sets RCC_D2CCIP1R register, DFSDM1SEL field.
+pub enum DfsdmSrc {
+    /// rcc_pclk2 is selected as DFSDM1 Clk kernel clock (default after reset)
+    Pclk2 = 0,
+    /// sys_ck clock is selected as DFSDM1 Clk kernel clock
+    Sysclk = 1,
+}
+
 #[derive(Clone, Copy, PartialEq)]
 #[repr(u8)]
 /// Clock divider for the HSI. See RCC_CR register, HSIDIV field.
@@ -307,6 +317,8 @@ pub struct Clocks {
     pub sai1_src: SaiSrc,
     /// SAI2 and SAI3 kernel clock source selection
     pub sai23_src: SaiSrc,
+    /// DFSDM1 kernel clock source selection
+    pub dfsdm1_src: DfsdmSrc,
 }
 
 impl Clocks {
@@ -458,6 +470,13 @@ impl Clocks {
         rcc.d3cfgr
             .modify(|_, w| unsafe { w.d3ppre().bits(self.d3_prescaler as u8) });
 
+        #[cfg(not(feature = "h7b3"))]
+        rcc.d2ccip1r.modify(|_, w| unsafe {
+            w.sai1sel().bits(self.sai1_src as u8);
+            w.sai23sel().bits(self.sai23_src as u8);
+            w.dfsdm1sel().bit(self.dfsdm1_src as u8 != 0)
+        });
+
         // Set USART2 to HSI, and USB to HSI48. Temp hardcoded.
         // todo: Add config enums for these, and add them as Clocks fields.
         #[cfg(not(feature = "h7b3"))]
@@ -600,12 +619,6 @@ impl Clocks {
             }
         }
 
-        #[cfg(not(feature = "h7b3"))]
-        rcc.d2ccip1r.modify(|_, w| unsafe {
-            w.sai1sel().bits(self.sai1_src as u8);
-            w.sai23sel().bits(self.sai23_src as u8)
-        });
-
         if self.hsi48_on {
             rcc.cr.modify(|_, w| w.hsi48on().set_bit());
             while rcc.cr.read().hsi48rdy().bit_is_clear() {}
@@ -775,6 +788,30 @@ impl Clocks {
         }
     }
 
+    /// Get the SAI1 audio clock freq
+    pub fn sai1_speed(&self) -> u32 {
+        let pll_src = match self.input_src {
+            InputSrc::Pll1(pll_src) => pll_src,
+            InputSrc::Csi => PllSrc::Csi,
+            InputSrc::Hsi(div) => PllSrc::Hsi(div),
+            InputSrc::Hse(freq) => PllSrc::Hse(freq),
+        };
+
+        match self.sai1_src {
+            SaiSrc::Pll1Q => {
+                self.pll_input_speed(pll_src, 1) * self.pll1.divn as u32 / self.pll1.divq as u32
+            }
+            SaiSrc::Pll2P => {
+                self.pll_input_speed(pll_src, 1) * self.pll2.divn as u32 / self.pll2.divp as u32
+            }
+            SaiSrc::Pll3P => {
+                self.pll_input_speed(pll_src, 1) * self.pll3.divn as u32 / self.pll3.divp as u32
+            }
+            SaiSrc::I2sCkin => unimplemented!(),
+            SaiSrc::PerClk => unimplemented!(),
+        }
+    }
+
     pub fn validate_speeds(&self) -> Result<(), SpeedError> {
         // todo: This depends on variant
         // let max_clock = 550_000_000;
@@ -857,8 +894,8 @@ impl Clocks {
 }
 
 impl Default for Clocks {
-    /// This default configures clocks with the HSI, and a 240Mhz sysclock speed.
-    /// Peripheral clocks are all set to 112.5Mhz, with timer clocks at 225Mhz.
+    /// This default configures clocks with the HSI, and a 400Mhz sysclock speed.
+    /// Peripheral and timer clocks are set to 100Mhz or 200Mhz, depending on their limits.
     /// HSE output is not bypassed.
     /// todo: Not 248Mhz due to an issue with wait states on Table 17 not showing higher than 225Mhz
     /// // todo HCLK unless in VOS0.
@@ -888,6 +925,7 @@ impl Default for Clocks {
             vos_range: VosRange::VOS1,
             sai1_src: SaiSrc::Pll1Q,
             sai23_src: SaiSrc::Pll1Q,
+            dfsdm1_src: DfsdmSrc::Pclk2,
         }
     }
 }
