@@ -79,6 +79,12 @@ pub enum DmaInput {
     Adc5 = 39,
     Sai1A = 108,
     Sai1B = 109,
+    // todo: These SAI2 values are bogus; can't find on G4 DMA mux.
+    Sai2A = 203,
+    Sai2B = 204,
+    // todo: Can't find DFSDM periph on G4 rm: These assigned values are bogus.
+    Dfsdm1F0 = 200,
+    Dfsdm1F1 = 201,
 }
 
 // todo: Trigger, synchronization etc mappings. Perhaps DmaTrigger, DmaSync enums etc.
@@ -123,6 +129,10 @@ pub enum DmaInput {
     Sai1B = 88,
     Sai2A = 89,
     Sai2B = 90,
+    Dfsdm1F0 = 101,
+    Dfsdm1F1 = 102,
+    Dfsdm1F2 = 103,
+    Dfsdm1F3 = 104,
 }
 
 impl DmaInput {
@@ -165,6 +175,11 @@ impl DmaInput {
             // Self::Adc3 => 37,
             // Self::Adc4 => 38,
             // Self::Adc5 => 39,
+            // Note: Sai1 appears to be DMA2 only.
+            Self::Sai2A => DmaChannel::C6,
+            Self::Sai2B => DmaChannel::C7,
+            Self::Dfsdm1F0 => DmaChannel::C5,
+            Self::Dfsdm1F1 => DmaChannel::C6,
             _ => unimplemented!(),
         }
     }
@@ -209,6 +224,8 @@ impl DmaInput {
             // Self::Adc3 => 37,
             // Self::Adc4 => 38,
             // Self::Adc5 => 39,
+            Self::Dfsdm1F0 => 0b0000,
+            Self::Dfsdm1F1 => 0b0000,
             _ => unimplemented!(),
         }
     }
@@ -1083,7 +1100,8 @@ where
         atomic::compiler_fence(Ordering::SeqCst);
     }
 
-    #[cfg(not(feature = "h7"))]
+    // todo: G0 removed from this fn due to a bug introduced in PAC 0.13
+    #[cfg(not(any(feature = "h7", feature = "g0")))]
     pub fn transfer_is_complete(&mut self, channel: DmaChannel) -> bool {
         let isr_val = self.regs.isr.read();
         match channel {
@@ -1353,7 +1371,8 @@ where
                         DmaInterrupt::FifoError => self.regs.hifcr.write(|w| w.cfeif7().set_bit()),
                     }
                 }
-            } else {
+                // todo: G0 PAC 0.14 had a reversion where these flags used to work, but now don't.
+            } else if #[cfg(not(feature = "g0"))] {
                 self.regs.ifcr.write(|w| match channel {
                     DmaChannel::C1 => match interrupt {
                         DmaInterrupt::TransferError => w.cteif1().set_bit(),
