@@ -46,7 +46,7 @@
 //! use stm32_hal2::{
 //!     clocks::Clocks,
 //!     gpio::{Pin, Port, PinMode, OutputType},
-//!     i2c::{I2c, I2cDevice},
+//!     i2c::I2c,
 //!     low_power,
 //!     pac,
 //!     timer::{Timer, TimerInterrupt},
@@ -71,7 +71,7 @@
 //!     let mut sda = Pin::new(Port::B, 7, PinMode::Alt(4));
 //!     sda.output_type(OutputType::OpenDrain);
 //!
-//!     let i2c = I2c::new(dp.I2C1, I2cDevice::One, Default::default(), &clock_cfg);
+//!     let i2c = I2c::new(dp.I2C1, Default::default(), &clock_cfg);
 //!
 //!     loop {
 //!         i2c.write(0x50, &[1, 2, 3]);
@@ -114,6 +114,8 @@
 // todo oncoe this is in stable.
 // #![feature(doc_cfg)]
 
+// todo: H7B3 has too many changes in v14 PAC; not supporting at this time. (2021-10-07)
+
 #[cfg(not(any(
     feature = "f301",
     feature = "f302",
@@ -362,6 +364,21 @@ pub mod crc;
 // WB doesn't have a DAC. Some G0 variants do - add it! Most F4 variants have it, some don't
 pub mod dac;
 
+#[cfg(not(any(
+    feature = "f3",
+    feature = "f4",
+    feature = "l4x1",
+    feature = "l4x2",
+    feature = "l412",
+    feature = "l4x3",
+    feature = "g0",
+    feature = "g4",
+    feature = "wb",
+    feature = "wl",
+    feature = "l5", // todo: L5 just doesn't have the same PAC layout; doesn't use clusters.
+)))]
+pub mod dfsdm;
+
 // todo: G0 missing many DMA registers like CCR?
 // todo: F4 needs some mods. So, only working on L4 and G4.
 // todo: L5 has a PAC bug on CCR registers past 1.
@@ -431,15 +448,17 @@ pub mod usart;
 cfg_if::cfg_if! {
     if #[cfg(all(
         feature = "usb",
-        any(
-            feature = "f303",
-            feature = "l4x2",
-            feature = "l412",
-            feature = "l4x3",
-            feature = "l5",
-            feature = "g4",
-            feature = "wb",
-        )
+        all(
+            any(
+                feature = "f303",
+                feature = "l4x2",
+                feature = "l412",
+                feature = "l4x3",
+                feature = "l5",
+                feature = "g4",
+                feature = "wb",
+            ),
+        not(feature = "g4a1"))
     ))] {
         pub mod usb;
     } else if #[cfg(all(
@@ -514,15 +533,19 @@ pub fn debug_workaround() {
         let dbgmcu = unsafe { &(*pac::DBGMCU::ptr()) };
 
         cfg_if::cfg_if! {
-            if #[cfg(not(feature = "h7"))] {
+            if #[cfg(all(feature = "h7", not(any(feature = "h747cm4", feature = "h747cm7"))))] {
+                dbgmcu.cr.modify(|_, w| w.dbgsleep_d1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstop_d1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstby_d1().set_bit());
+            } else if #[cfg(feature = "h7")] {
+                dbgmcu.cr.modify(|_, w| w.dbgslpd1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstpd1().set_bit());
+                dbgmcu.cr.modify(|_, w| w.dbgstbd1().set_bit());
+            } else {
                 #[cfg(not(feature = "l5"))]
                 dbgmcu.cr.modify(|_, w| w.dbg_sleep().set_bit());
                 dbgmcu.cr.modify(|_, w| w.dbg_stop().set_bit());
                 dbgmcu.cr.modify(|_, w| w.dbg_standby().set_bit());
-            } else {
-                dbgmcu.cr.modify(|_, w| w.dbgsleep_d1().set_bit());
-                dbgmcu.cr.modify(|_, w| w.dbgstop_d1().set_bit());
-                dbgmcu.cr.modify(|_, w| w.dbgstby_d1().set_bit());
             }
         }
     });
